#!/bin/bash

DRIVER_SOURCE_PATH=HTTP_INTEL_DRIVERS

PREFIX="/opt/gfx-test-$(date +%y%m%d)"
PKG_CONFIG_PATH=$PREFIX/lib/pkgconfig

MAKE="make -j3"

REPOS="\
git://git.freedesktop.org/git/xorg/util/macros \
git://git.freedesktop.org/git/xorg/proto/x11proto \
git://git.freedesktop.org/git/xorg/proto/damageproto \
git://git.freedesktop.org/git/xorg/proto/xextproto \
git://git.freedesktop.org/git/xorg/proto/fontsproto \
git://git.freedesktop.org/git/xorg/proto/videoproto \
git://git.freedesktop.org/git/xorg/proto/renderproto \
git://git.freedesktop.org/git/xorg/proto/inputproto \
git://git.freedesktop.org/git/xorg/proto/xf86vidmodeproto \
git://git.freedesktop.org/git/xorg/proto/xf86dgaproto \
git://git.freedesktop.org/git/xorg/proto/xf86driproto \
git://git.freedesktop.org/git/xorg/proto/xcmiscproto \
git://git.freedesktop.org/git/xorg/proto/scrnsaverproto \
git://git.freedesktop.org/git/xorg/proto/bigreqsproto \
git://git.freedesktop.org/git/xorg/proto/resourceproto \
git://git.freedesktop.org/git/xorg/proto/compositeproto \
git://git.freedesktop.org/git/xorg/proto/fixesproto \
git://git.freedesktop.org/git/xorg/proto/evieproto \
git://git.freedesktop.org/git/xorg/proto/kbproto \
git://git.freedesktop.org/git/xorg/lib/libxtrans \
git://git.freedesktop.org/git/xorg/lib/libX11 \
git://git.freedesktop.org/git/xorg/lib/libXext \
git://git.freedesktop.org/git/xorg/lib/libxkbfile \
git://git.freedesktop.org/git/xorg/lib/libfontenc \
git://git.freedesktop.org/git/xorg/lib/libXfont \
git://git.freedesktop.org/git/xorg/lib/libXfixes \
git://git.freedesktop.org/git/xorg/lib/libXdamage \
git://git.freedesktop.org/git/xorg/lib/libXv \
git://git.freedesktop.org/git/xorg/lib/libXvMC \
git://git.freedesktop.org/git/xorg/lib/libXxf86vm \
git://git.freedesktop.org/git/xorg/lib/libXinerama \
git://git.freedesktop.org/git/xorg/proto/dri2proto \
git://git.freedesktop.org/git/xorg/proto/glproto \
git://git.freedesktop.org/git/xorg/lib/libpciaccess \
git://git.freedesktop.org/git/pixman \
git://git.freedesktop.org/git/xcb/proto \
git://git.freedesktop.org/git/xcb/pthread-stubs \
git://git.freedesktop.org/git/xcb/libxcb \
git://git.freedesktop.org/git/xorg/proto/randrproto \
git://git.freedesktop.org/git/mesa/drm \
git://git.freedesktop.org/git/mesa/mesa \
git://git.freedesktop.org/git/xorg/xserver \
git://git.freedesktop.org/git/xorg/driver/xf86-input-mouse \
git://git.freedesktop.org/git/xorg/driver/xf86-input-keyboard \
git://git.freedesktop.org/git/xorg/driver/xf86-video-intel \
git://git.freedesktop.org/git/xorg/driver/xf86-input-evdev \
git://git.freedesktop.org/git/xorg/driver/xf86-video-vesa \
git://git.freedesktop.org/git/xorg/driver/xf86-video-fbdev"

# Using other method (below), recommended by "Making the keyboard work" 
# on http://www.x.org/wiki/Development/git
#git://git.freedesktop.org/git/xkeyboard-config

# This is huge... already installed from .tar.bz2 from http://www.intellinuxgraphics.com
#git://git.kernel.org/pub/scm/linux/kernel/git/anholt/drm-intel.git

# ---------------------------------------------------------------------
# Order of installation:
# ---------------------------------------------------------------------

# NOTES: git repository names are different from http repository names
# x11proto is xproto
# fixesproto is FIXES
# libxtrans in xtrans
pre_modules="\
fontsproto \
x11proto \
xproto \
xextproto \
videoproto \
renderproto \
inputproto \
damageproto \
xf86vidmodeproto \
xf86dgaproto \
xf86driproto \
xcmiscproto \
scrnsaverproto \
bigreqsproto \
resourceproto \
compositeproto \
resourceproto \
evieproto \
kbproto \
proto \
dri2proto \
glproto \
randrproto \
pthread-stubs \
pixman \
libxcb \
libX11 \
libXext \
libxtrans \
xtrans \
libxkbfile \
libfontenc \
libXfont \
libXv \
libXvMC \
libXxf86vm \
libXinerama \
libXfixes \
libXdamage \
libpciaccess"

# REMOVED fixesproto (FIXES) because it says:
# checking for XEXTENSIONS... configure: error: Package requirements (xextensions) were not met:
# No package 'xextensions' found
# I couldn't easily find xextensions, see this link:
# http://www.nabble.com/xextensions-and-xextproto-install-the-same-files-td12677331.html
# He claims "This package was replaced with xextproto in April."
# Then why does "fixes" still depend on "xextensions" instead of "xextproto".

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# core_modules are: drm, mesa, xserver.
# They need special treatment, see below.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

post_modules="\
xf86-input-evdev \
xf86-input-mouse \
xf86-input-keyboard \
xf86-video-vesa \
xf86-video-fbdev \
xf86-video-intel"

# ---------------------------------------------------------------------
# Functions:
# ---------------------------------------------------------------------

clone()
{
        for repo in $REPOS; do
                echo "Cloning $repo";
                git clone $repo;
        done
}

update_modules()
{
        for module in $pre_modules; do
                cd $module
		echo checking $module
                git pull
                cd ..
        done

        for module in $post_modules; do
                cd $module
		echo checking $module
                git pull
                cd ..
        done
}

init()
{
    init_success=0

    echo "NOTE: If SUCCESS isn't printed, then an error occured.";
    echo "Sending output to intel_install_INIT_*.txt...";

    { # Begin code block.

        for ix in `find -maxdepth 1 -type d`; do
            # Remove leading ./
            i=${ix#*/}
            if [[ $i == *macros* ]]; then
                cd $i
                echo ======================
                echo configuring $i
                echo ======================
                ./autogen.sh --prefix="$PREFIX";
                if [ $? -ne 0 ]; then echo "Failed to configure $i."; exit; fi        
                echo ======================
                echo building $i
                echo ======================
                ($MAKE);
                if [ $? -ne 0 ]; then echo "Failed to build $i."; exit; fi        
                echo ======================
                echo installing $i
                echo ======================
                make install
                if [ $? -ne 0 ]; then echo "Failed to install $i."; exit; fi        
                cd ..

                (( init_success++ ))
                break  # we're done!
            fi
        done
    # Redirects output of everything in block to file.
    } &> "intel_install_INIT_$(date +%y%m%d-%H:%M).txt"  
    echo "Output copied to intel_install_INIT_$(date +%y%m%d-%H:%M).txt"

    echo ======================
    if [ $init_success -ne 1 ]
    then
        echo "FAILURE: check log file! $init_success";
    else
        echo "SUCCESS: init complete.";
    fi
    echo ======================
}

build ()
{
    echo "NOTE: If SUCCESS isn't printed, then an error occured.";
    echo "Sending output to intel_install_BUILD_*.txt...";

    { # Begin code block.

        export ACLOCAL="aclocal -I $PREFIX/share/aclocal"
        export PKG_CONFIG_PATH="$PREFIX/lib/pkgconfig"

# pre_modules
        pre_module_count=0
        pre_module_install_count=0
        for mx in $pre_modules; do
            (( pre_module_count++ ))
            echo Does $mx have a source directory?
            for ix in `find -maxdepth 1 -type d`; do
                # Remove leading ./
                i=${ix#*/}

                # Skip directories that are built later
                if [[ $i == $mx* ]]; then
                    cd $i
                    echo ======================
                    echo configuring $i
                    echo ======================
                    ./autogen.sh --prefix="$PREFIX";
                    if [ $? -ne 0 ]; then echo "Failed to configure $i."; exit; fi        
                    echo ======================
                    echo building $i
                    echo ======================
                    ($MAKE);
                    if [ $? -ne 0 ]; then echo "Failed to build $i."; exit; fi        
                    echo ======================
                    echo installing $i
                    echo ======================
                    make install
                    if [ $? -ne 0 ]; then echo "Failed to install $i."; exit; fi        
                    cd ..
                    (( pre_module_install_count++ ))
                    break  # Go to next in list of pre_modules
                fi
            done
        done

        if [ $pre_module_install_count -ne $pre_module_count ]
        then
            echo ======================
            echo "NOTE: mismatch in number of pre_modules: $pre_module_install_count != $pre_module_count";
            echo ======================
        fi

# build drm
        for ix in `find -maxdepth 1 -type d`; do
            # Remove leading ./
            i=${ix#*/}
            if [[ $i == *drm* ]]; then
                cd $i
                echo ======================
                echo configuring $i
                echo ======================
                ./autogen.sh --prefix="$PREFIX"
                if [ $? -ne 0 ]; then echo "Failed to configure $i."; exit; fi
                echo ======================
                echo building $i
                echo ======================
                ($MAKE)
                if [ $? -ne 0 ]; then echo "Failed to build $i."; exit; fi
                make -C linux-core
                # assuming you're on Linux, otherwise use bsd-core
                echo ======================
                echo installing $i
                echo ======================
                make install
                if [ $? -ne 0 ]; then echo "Failed to install $i."; exit; fi
                cd ..

                break  # we're done!
            fi
        done
#build mesa
        for ix in `find -maxdepth 1 -type d`; do
            # Remove leading ./
            i=${ix#*/}
            if [[ $i == *mesa* ]]; then
                cd $i
                echo ======================
                echo configuring $i
                echo ======================
                ./autogen.sh --prefix=$PREFIX --with-driver=dri --disable-glut
                if [ $? -ne 0 ]; then echo "Failed to configure $i."; exit; fi
                echo ======================
                echo building $i
                echo ======================
                ($MAKE)
                if [ $? -ne 0 ]; then echo "Failed to build $i."; exit; fi
                echo ======================
                echo installing $i
                echo ======================
                make install
                if [ $? -ne 0 ]; then echo "Failed to install $i."; exit; fi
                echo ======================
                echo installing $i demos
                echo ======================
                mkdir -p $PREFIX/bin
                if [ $? -ne 0 ]; then echo "Failed to install $i (bin)."; exit; fi
                install -m755 progs/xdemos/{glxinfo,glxgears} $PREFIX/bin/
                if [ $? -ne 0 ]; then echo "Failed to install $i demos."; exit; fi
                cd ..

                break  # we're done!
            fi
        done
#build xserver
        for ix in `find -maxdepth 1 -type d`; do
            # Remove leading ./
            i=${ix#*/}
            if [[ $i == *server* ]]; then
                cd $i
                echo ======================
                echo configuring $i
                echo ======================
                ./autogen.sh --prefix=$PREFIX --enable-builtin-fonts --with-xkb-path=/usr/share/X11/xkb
                if [ $? -ne 0 ]; then echo "Failed to configure $i."; exit; fi
                echo ======================
                echo building $i
                echo ======================
                ($MAKE)
                if [ $? -ne 0 ]; then echo "Failed to build $i."; exit; fi
                echo ======================
                echo installing $i
                echo ======================
                make install
                if [ $? -ne 0 ]; then echo "Failed to install $i."; exit; fi
                echo ======================
                echo installing $i, cont
                echo ======================
                chown root $PREFIX/bin/Xorg;
                if [ $? -ne 0 ]; then echo "Failed to install $i (chown)."; exit; fi
                chmod +s $PREFIX/bin/Xorg
                if [ $? -ne 0 ]; then echo "Failed to install $i (chmod)."; exit; fi
                cd ..

                # Suggested by "Making the keyboard work" on http://www.x.org/wiki/Development/git
                # Also --with-xkb-path=/usr/share/X11/xkb in autogen script.
                ln -s /usr/bin/xkbcomp $PREFIX/bin/xkbcomp
                if [ $? -ne 0 ]; then echo "Failed to install $i (ln -s)."; exit; fi

                break  # we're done!
            fi
        done

# post_modules
        post_module_count=0
        post_module_install_count=0
        for mx in $post_modules; do
            (( post_module_count++ ))
            for ix in `find -maxdepth 1 -type d`; do
                # Remove leading ./
                i=${ix#*/}

                # Skip directories that were built earlier
                if [[ $i == $mx* ]]; then
                    cd $i
                    echo ======================
                    echo configuring $i
                    echo ======================
                    ./autogen.sh --prefix="$PREFIX";
                    if [ $? -ne 0 ]; then echo "Failed to configure $i."; exit; fi        
                    echo ======================
                    echo building $i
                    echo ======================
                    ($MAKE);
                    if [ $? -ne 0 ]; then echo "Failed to build $i."; exit; fi        
                    echo ======================
                    echo installing $i
                    echo ======================
                    make install
                    if [ $? -ne 0 ]; then echo "Failed to install $i."; exit; fi        
                    cd ..

                    (( post_module_install_count++ ))
                    break  # Go to next in list of post_modules
                fi
            done
        done

    # Redirects output of everything in block to file.
    } &> "intel_install_BUILD_$(date +%y%m%d-%H:%M).txt"  
    echo "Output copied to intel_install_BUILD_$(date +%y%m%d-%H:%M).txt"

    echo ======================
    if [ $post_module_install_count -ne $post_module_count ]
    then
        echo "FAILURE: check log file! $post_module_install_count != $post_module_count";
    else
        echo "SUCCESS: build complete. $post_module_install_count == $post_module_count";
    fi
    echo ======================
}

case "$1" in
        init)
                cd $DRIVER_SOURCE_PATH
                init
                cd ..
                ;;
        build)
                cd $DRIVER_SOURCE_PATH
                build
                cd ..
                ;;
        clone)
                cd $DRIVER_SOURCE_PATH
                clone
                cd ..
                ;;
        update)
                cd $DRIVER_SOURCE_PATH
                update_modules
                cd ..
                ;;
        *)
                cd $DRIVER_SOURCE_PATH
                echo "Usage: $0 init | build | clone | update"
                cd ..
                exit 3
esac

